import { CoreManager, CoreManagerInstance } from './CoreManager';
import decode from './decode';
import encode from './encode';
import ParseError from './ParseError';
import ParsePromise from './ParsePromise';

export class ParseHooksFacade {

  constructor(coreManager) {
    this._CoreManager = coreManager;
    this._CoreManager.setHooksFacade(this);
    this._CoreManager.setHooksController(new DefaultController(this._CoreManager));
  }

  getFunctions() {
    return this._CoreManager.getHooksController().get("functions");
  }

  getTriggers() {
    return this._CoreManager.getHooksController().get("triggers");
  }

  getFunction(name) {
    return this._CoreManager.getHooksController().get("functions", name);
  }

  getTrigger(className, triggerName) {
    return this._CoreManager.getHooksController().get("triggers", className, triggerName);
  }

  createFunction(functionName, url) {
    return this.create({ functionName: functionName, url: url });
  }

  createTrigger(className, triggerName, url) {
    return this.create({ className: className, triggerName: triggerName, url: url });
  }

  create(hook) {
    return this._CoreManager.getHooksController().create(hook);
  }

  updateFunction(functionName, url) {
    return this.update({ functionName: functionName, url: url });
  }

  updateTrigger(className, triggerName, url) {
    return this.update({ className: className, triggerName: triggerName, url: url });
  }

  update(hook) {
    return this._CoreManager.getHooksController().update(hook);
  }

  removeFunction(functionName) {
    return this.remove({ functionName: functionName });
  }

  removeTrigger(className, triggerName) {
    return this.remove({ className: className, triggerName: triggerName });
  }

  remove(hook) {
    return this._CoreManager.getHooksController().remove(hook);
  }
}

class DefaultController {

  constructor(coreManager) {
    this._CoreManager = coreManager;
  }

  get(type, functionName, triggerName) {
    var url = "/hooks/" + type;
    if (functionName) {
      url += "/" + functionName;
      if (triggerName) {
        url += "/" + triggerName;
      }
    }
    return this.sendRequest("GET", url);
  }

  create(hook) {
    var url;
    if (hook.functionName && hook.url) {
      url = "/hooks/functions";
    } else if (hook.className && hook.triggerName && hook.url) {
      url = "/hooks/triggers";
    } else {
      return Promise.reject({ error: 'invalid hook declaration', code: 143 });
    }
    return this.sendRequest("POST", url, hook);
  }

  remove(hook) {
    var url;
    if (hook.functionName) {
      url = "/hooks/functions/" + hook.functionName;
      delete hook.functionName;
    } else if (hook.className && hook.triggerName) {
      url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
      delete hook.className;
      delete hook.triggerName;
    } else {
      return Promise.reject({ error: 'invalid hook declaration', code: 143 });
    }
    return this.sendRequest("PUT", url, { "__op": "Delete" });
  }

  update(hook) {
    var url;
    if (hook.functionName && hook.url) {
      url = "/hooks/functions/" + hook.functionName;
      delete hook.functionName;
    } else if (hook.className && hook.triggerName && hook.url) {
      url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
      delete hook.className;
      delete hook.triggerName;
    } else {
      return Promise.reject({ error: 'invalid hook declaration', code: 143 });
    }
    return this.sendRequest('PUT', url, hook);
  }

  sendRequest(method, url, body) {
    return this._CoreManager.getRESTController().request(method, url, body, { useMasterKey: true }).then(res => {
      var decoded = decode(res, this._CoreManager);
      if (decoded) {
        return ParsePromise.as(decoded);
      }
      return ParsePromise.error(new ParseError(ParseError.INVALID_JSON, 'The server returned an invalid response.'));
    });
  }
}

export const ParseHooks = new ParseHooksFacade(CoreManager);

export default ParseHooks;