/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

import EventEmitter from './EventEmitter';
import LiveQueryClient from './LiveQueryClient';
import { CoreManager, CoreManagerInstance } from './CoreManager';
import ParsePromise from './ParsePromise';

function getSessionToken(coreManager) {
  const controller = coreManager.getUserController();
  return controller.currentUserAsync().then(currentUser => {
    return currentUser ? currentUser.getSessionToken() : undefined;
  });
}

function getLiveQueryClient(coreManager) {
  return coreManager.getLiveQueryController().getDefaultLiveQueryClient();
}

/**
 *
 * We expose three events to help you monitor the status of the WebSocket connection:
 *
 * <p>Open - When we establish the WebSocket connection to the LiveQuery server, you'll get this event.
 * 
 * <pre>
 * Parse.LiveQuery.on('open', () => {
 * 
 * });</pre></p>
 *
 * <p>Close - When we lose the WebSocket connection to the LiveQuery server, you'll get this event.
 * 
 * <pre>
 * Parse.LiveQuery.on('close', () => {
 * 
 * });</pre></p>
 *
 * <p>Error - When some network error or LiveQuery server error happens, you'll get this event.
 * 
 * <pre>
 * Parse.LiveQuery.on('error', (error) => {
 * 
 * });</pre></p>
 * 
 * @class Parse.LiveQuery
 * @static
 * 
 */
export class LiveQueryFacade extends EventEmitter {

  constructor(coreManager) {
    super();
    this._CoreManager = coreManager;
    this.on('error', () => {});
    this._CoreManager.setLiveQueryController(new DefaultLiveQueryController(this._CoreManager, this));
  }

  open() {
    const LiveQueryController = this._CoreManager.getLiveQueryController();
    LiveQueryController.open();
  }

  close() {
    const LiveQueryController = this._CoreManager.getLiveQueryController();
    LiveQueryController.close();
  }
}

class DefaultLiveQueryController {

  constructor(coreManager, liveQuery) {
    this._CoreManager = coreManager;
    this._LiveQuery = liveQuery;
  }

  setDefaultLiveQueryClient(liveQueryClient) {
    this.defaultLiveQueryClient = liveQueryClient;
  }
  getDefaultLiveQueryClient() {
    if (this.defaultLiveQueryClient) {
      return ParsePromise.as(this.defaultLiveQueryClient);
    }

    return getSessionToken(this._CoreManager).then(sessionToken => {
      let liveQueryServerURL = this._CoreManager.get('LIVEQUERY_SERVER_URL');

      if (liveQueryServerURL && liveQueryServerURL.indexOf('ws') !== 0) {
        throw new Error('You need to set a proper Parse LiveQuery server url before using LiveQueryClient');
      }

      // If we can not find Parse.liveQueryServerURL, we try to extract it from Parse.serverURL
      if (!liveQueryServerURL) {
        const tempServerURL = this._CoreManager.get('SERVER_URL');
        let protocol = 'ws://';
        // If Parse is being served over SSL/HTTPS, ensure LiveQuery Server uses 'wss://' prefix
        if (tempServerURL.indexOf('https') === 0) {
          protocol = 'wss://';
        }
        const host = tempServerURL.replace(/^https?:\/\//, '');
        liveQueryServerURL = protocol + host;
        this._CoreManager.set('LIVEQUERY_SERVER_URL', liveQueryServerURL);
      }

      const applicationId = this._CoreManager.get('APPLICATION_ID');
      const javascriptKey = this._CoreManager.get('JAVASCRIPT_KEY');
      const masterKey = this._CoreManager.get('MASTER_KEY');
      // Get currentUser sessionToken if possible
      this.defaultLiveQueryClient = new LiveQueryClient({
        applicationId,
        serverURL: liveQueryServerURL,
        javascriptKey,
        masterKey,
        sessionToken,
        coreManager: this._CoreManager
      });
      // Register a default onError callback to make sure we do not crash on error
      // Cannot create these events on a nested way because of EventEmiiter from React Native
      this.defaultLiveQueryClient.on('error', error => {
        this._LiveQuery.emit('error', error);
      });
      this.defaultLiveQueryClient.on('open', () => {
        this._LiveQuery.emit('open');
      });
      this.defaultLiveQueryClient.on('close', () => {
        this._LiveQuery.emit('close');
      });

      return this.defaultLiveQueryClient;
    });
  }
  open() {
    getLiveQueryClient(this._CoreManager).then(liveQueryClient => {
      this.resolve(liveQueryClient.open());
    });
  }
  close() {
    getLiveQueryClient(this._CoreManager).then(liveQueryClient => {
      this.resolve(liveQueryClient.close());
    });
  }
  subscribe(query) {
    let subscriptionWrap = new EventEmitter();

    getLiveQueryClient(this._CoreManager).then(liveQueryClient => {
      if (liveQueryClient.shouldOpen()) {
        liveQueryClient.open();
      }
      let promiseSessionToken = getSessionToken(this._CoreManager);
      // new event emitter
      return promiseSessionToken.then(sessionToken => {

        let subscription = liveQueryClient.subscribe(query, sessionToken);
        // enter, leave create, etc

        subscriptionWrap.id = subscription.id;
        subscriptionWrap.query = subscription.query;
        subscriptionWrap.sessionToken = subscription.sessionToken;
        subscriptionWrap.unsubscribe = subscription.unsubscribe;
        // Cannot create these events on a nested way because of EventEmiiter from React Native
        subscription.on('open', () => {
          subscriptionWrap.emit('open');
        });
        subscription.on('create', object => {
          subscriptionWrap.emit('create', object);
        });
        subscription.on('update', object => {
          subscriptionWrap.emit('update', object);
        });
        subscription.on('enter', object => {
          subscriptionWrap.emit('enter', object);
        });
        subscription.on('leave', object => {
          subscriptionWrap.emit('leave', object);
        });
        subscription.on('delete', object => {
          subscriptionWrap.emit('delete', object);
        });
        subscription.on('close', object => {
          subscriptionWrap.emit('close', object);
        });
        subscription.on('error', object => {
          subscriptionWrap.emit('error', object);
        });

        this.resolve();
      });
    });
    return subscriptionWrap;
  }
  unsubscribe(subscription) {
    getLiveQueryClient(this._CoreManager).then(liveQueryClient => {
      this.resolve(liveQueryClient.unsubscribe(subscription));
    });
  }
  _clearCachedDefaultClient() {
    this.defaultLiveQueryClient = null;
  }
}

const LiveQuery = new LiveQueryFacade(CoreManager);

export default LiveQuery;