/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

import { CoreManagerInstance, CoreManager } from './CoreManager';
import isRevocableSession from './isRevocableSession';
import ParseObject from './ParseObject';
import ParsePromise from './ParsePromise';
import ParseUser from './ParseUser';

export function makeParseSession(coreManager) {
  return class extends ParseSession {
    constructor(attributes) {
      super(attributes, coreManager);
    }
  };
}

/**
 * <p>A Parse.Session object is a local representation of a revocable session.
 * This class is a subclass of a Parse.Object, and retains the same
 * functionality of a Parse.Object.</p>
 * @alias Parse.Session
 * @extends Parse.Object
 */
export class ParseSession extends ParseObject {
  /**
   * 
   * @param {Object} attributes The initial set of data to store in the user.
   */
  constructor(attributes, coreManager) {
    super('_Session', undefined, undefined, coreManager);
    if (attributes && typeof attributes === 'object') {
      if (!this.set(attributes || {})) {
        throw new Error('Can\'t create an invalid Session');
      }
    }
  }

  static readOnlyAttributes() {
    return ['createdWith', 'expiresAt', 'installationId', 'restricted', 'sessionToken', 'user'];
  }

  /**
   * Returns the session token string.
    * @return {String}
   */
  getSessionToken() {
    const token = this.get('sessionToken');
    if (typeof token === 'string') {
      return token;
    }
    return '';
  }
}

export class ParseSessionFacade {

  constructor(coreManager) {
    if (coreManager.getSessionFacade()) {
      throw new Error('Cannot set ParseSessionFacade mutliple times to coreManager');
    }
    this._CoreManager = coreManager;
    this._CoreManager.setSessionFacade(this);
    this.Instance = makeParseSession(this._CoreManager);

    this._CoreManager.setSessionController(new DefaultController(this._CoreManager));
    this._CoreManager.getObjectFacade().registerSubclass('_Session', this.Instance);
  }

  /**
   * Retrieves the Session object for the currently logged in session.
    * @static
   * @return {Parse.Promise} A promise that is resolved with the Parse.Session
   *   object after it has been fetched. If there is no current user, the
   *   promise will be rejected.
   */
  current(options) {
    options = options || {};
    var controller = this._CoreManager.getSessionController();

    var sessionOptions = {};
    if (options.hasOwnProperty('useMasterKey')) {
      sessionOptions.useMasterKey = options.useMasterKey;
    }
    return this._CoreManager.getUserFacade().currentAsync().then(user => {
      if (!user) {
        return ParsePromise.error('There is no current user.');
      }
      user.getSessionToken();

      sessionOptions.sessionToken = user.getSessionToken();
      return controller.getSession(sessionOptions);
    });
  }

  /**
   * Determines whether the current session token is revocable.
   * This method is useful for migrating Express.js or Node.js web apps to
   * use revocable sessions. If you are migrating an app that uses the Parse
   * SDK in the browser only, please use Parse.User.enableRevocableSession()
   * instead, so that sessions can be automatically upgraded.
    * @static
   * @return {Boolean}
   */
  isCurrentSessionRevocable() {
    var currentUser = this._CoreManager.getUserFacade().current();
    if (currentUser) {
      return isRevocableSession(currentUser.getSessionToken() || '');
    }
    return false;
  }
}

class DefaultController {

  constructor(coreManager) {
    this._CoreManager = coreManager;
  }

  getSession(options) {
    var RESTController = this._CoreManager.getRESTController();
    var session = new ParseSession(undefined, this._CoreManager);

    return RESTController.request('GET', 'sessions/me', {}, options).then(sessionData => {
      session._finishFetch(sessionData);
      session._setExisted(true);
      return session;
    });
  }
}

const _ParseSessionFacade = new ParseSessionFacade(CoreManager);

export default ParseSession;