import { CoreManager, CoreManagerInstance } from './CoreManager';
import decode from './decode';
import encode from './encode';
import ParseError from './ParseError';
import ParsePromise from './ParsePromise';

export class ParseTasksFacade {
  constructor(coreManager) {
    this._CoreManager = coreManager;
    this._CoreManager.setTasksFacade(this);
    this._CoreManager.setTasksController(new DefaultController(this._CoreManager));
  }

  getFunctions() {
    return this._CoreManager.getTasksController().get("functions");
  }

  getTriggers() {
    return this._CoreManager.getTasksController().get("triggers");
  }

  getFunction(name) {
    return this._CoreManager.getTasksController().get("functions", name);
  }

  getTrigger(className, triggerName) {
    return this._CoreManager.getTasksController().get("triggers", className, triggerName);
  }

  createFunction(functionName, taskDefinition, taskRepeat) {
    return this.create({ functionName: functionName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  }

  createTrigger(className, triggerName, taskDefinition, taskRepeat) {
    return this.create({ className: className, triggerName: triggerName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  }

  create(task) {
    return this._CoreManager.getTasksController().create(task);
  }

  updateFunction(functionName, taskDefinition, taskRepeat) {
    return this.update({ functionName: functionName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  }

  updateTrigger(className, triggerName, taskDefinition, taskRepeat) {
    return this.update({ className: className, triggerName: triggerName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  }

  update(task) {
    return this._CoreManager.getTasksController().update(task);
  }

  removeFunction(functionName) {
    return this.remove({ functionName: functionName });
  }

  removeTrigger(className, triggerName) {
    return this.remove({ className: className, triggerName: triggerName });
  }

  remove(task) {
    return this._CoreManager.getTasksController().remove(task);
  }
}

class DefaultController {

  constructor(coreManager) {
    this._CoreManager = coreManager;
  }

  get(type, functionName, triggerName) {
    var url = "/tasks/" + type;
    if (functionName) {
      url += "/" + functionName;
      if (triggerName) {
        url += "/" + triggerName;
      }
    }
    return this.sendRequest("GET", url);
  }

  create(task) {
    var url;
    if (task.functionName && task.taskDefinition) {
      url = "/tasks/functions";
    } else if (task.className && task.triggerName && task.taskDefinition) {
      url = "/tasks/triggers";
    } else {
      return Promise.reject({ error: 'invalid task declaration', code: 143 });
    }
    return this.sendRequest("POST", url, task);
  }

  remove(task) {
    var url;
    if (task.functionName) {
      url = "/tasks/functions/" + task.functionName;
      delete task.functionName;
    } else if (task.className && task.triggerName) {
      url = "/tasks/triggers/" + task.className + "/" + task.triggerName;
      delete task.className;
      delete task.triggerName;
    } else {
      return Promise.reject({ error: 'invalid task declaration', code: 143 });
    }
    return this.sendRequest("PUT", url, { "__op": "Delete" });
  }

  update(task) {
    var url;
    if (task.functionName && task.taskDefinition) {
      url = "/tasks/functions/" + task.functionName;
      delete task.functionName;
    } else if (task.className && task.triggerName && task.taskDefinition) {
      url = "/tasks/triggers/" + task.className + "/" + task.triggerName;
      delete task.className;
      delete task.triggerName;
    } else {
      return Promise.reject({ error: 'invalid task declaration', code: 143 });
    }
    return this.sendRequest('PUT', url, task);
  }

  sendRequest(method, url, body) {
    return this._CoreManager.getRESTController().request(method, url, body, { useMasterKey: true }).then(res => {
      var decoded = decode(res, this._CoreManager);
      if (decoded) {
        return ParsePromise.as(decoded);
      }
      return ParsePromise.error(new ParseError(ParseError.INVALID_JSON, 'The server returned an invalid response.'));
    });
  }
}

export const ParseTasks = new ParseTasksFacade(CoreManager);

export default ParseTasks;