/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

import * as ObjectStateMutations from './ObjectStateMutations';

export default class SingleInstanceStateController {

  constructor() {
    this.objectState = {};
  }

  getState(obj) {
    let classData = this.objectState[obj.className];
    if (classData) {
      return classData[obj.id] || null;
    }
    return null;
  }

  initializeState(obj, initial) {
    let state = this.getState(obj);
    if (state) {
      return state;
    }
    if (!this.objectState[obj.className]) {
      this.objectState[obj.className] = {};
    }
    if (!initial) {
      initial = ObjectStateMutations.defaultState();
    }
    state = this.objectState[obj.className][obj.id] = initial;
    return state;
  }

  removeState(obj) {
    let state = this.getState(obj);
    if (state === null) {
      return null;
    }
    delete this.objectState[obj.className][obj.id];
    return state;
  }

  getServerData(obj) {
    let state = this.getState(obj);
    if (state) {
      return state.serverData;
    }
    return {};
  }

  setServerData(obj, attributes) {
    let serverData = this.initializeState(obj).serverData;
    ObjectStateMutations.setServerData(serverData, attributes);
  }

  getPendingOps(obj) {
    let state = this.getState(obj);
    if (state) {
      return state.pendingOps;
    }
    return [{}];
  }

  setPendingOp(obj, attr, op) {
    let pendingOps = this.initializeState(obj).pendingOps;
    ObjectStateMutations.setPendingOp(pendingOps, attr, op);
  }

  pushPendingState(obj) {
    let pendingOps = this.initializeState(obj).pendingOps;
    ObjectStateMutations.pushPendingState(pendingOps);
  }

  popPendingState(obj) {
    let pendingOps = this.initializeState(obj).pendingOps;
    return ObjectStateMutations.popPendingState(pendingOps);
  }

  mergeFirstPendingState(obj) {
    let pendingOps = this.getPendingOps(obj);
    ObjectStateMutations.mergeFirstPendingState(pendingOps);
  }

  getObjectCache(obj) {
    let state = this.getState(obj);
    if (state) {
      return state.objectCache;
    }
    return {};
  }

  estimateAttribute(obj, attr) {
    let serverData = this.getServerData(obj);
    let pendingOps = this.getPendingOps(obj);
    return ObjectStateMutations.estimateAttribute(serverData, pendingOps, obj.className, obj.id, obj.coreManager, attr);
  }

  estimateAttributes(obj) {
    let serverData = this.getServerData(obj);
    let pendingOps = this.getPendingOps(obj);
    return ObjectStateMutations.estimateAttributes(serverData, pendingOps, obj.className, obj.id, obj.coreManager);
  }

  commitServerChanges(obj, changes) {
    let state = this.initializeState(obj);
    ObjectStateMutations.commitServerChanges(state.serverData, state.objectCache, changes);
  }

  enqueueTask(obj, task) {
    let state = this.initializeState(obj);
    return state.tasks.enqueue(task);
  }

  clearAllState() {
    this.objectState = {};
  }

  duplicateState(source, dest) {
    dest.id = source.id;
  }
}